import re
import subprocess

from enum import Enum

from simtools.Utilities.General import init_logging
from simtools.Utilities.RetryDecorator import retry

logger = init_logging("SLURM")


# From idmtools
class EntityStatus(Enum):
    CREATED = 'created'
    RUNNING = 'running'
    SUCCEEDED = 'succeeded'
    FAILED = 'failed'


SLURM_STATES = dict(
    BOOT_FAIL=EntityStatus.FAILED,
    CANCELLED=EntityStatus.FAILED,
    COMPLETED=EntityStatus.SUCCEEDED,
    DEADLINE=EntityStatus.FAILED,
    FAILED=EntityStatus.FAILED,
    OUT_OF_MEMORY=EntityStatus.FAILED,
    PENDING=EntityStatus.RUNNING,
    PREEMPTED=EntityStatus.FAILED,
    RUNNING=EntityStatus.RUNNING,
    REQUEUED=EntityStatus.RUNNING,
    RESIZING=EntityStatus.RUNNING,
    REVOKED=EntityStatus.FAILED,
    SUSPENDED=EntityStatus.FAILED,
    TIMEOUT=EntityStatus.FAILED
)


SBATCH_REGEX = re.compile('^[a-zA-Z ]+(?P<id>\d+)$')
SACCT_REGEX = re.compile('^(?P<id>[\w.\[\]-]+)\|(?P<state>[\w ]+)$')


def submit_job(script_path):
    result = subprocess.run(['sbatch', script_path], stdout=subprocess.PIPE)
    result = result.stdout.decode('utf-8').strip()
    job_id = SBATCH_REGEX.match(result).group('id')
    logger.info('Submitted slurm batch id: %s' % job_id)
    return job_id


def cancel_job(job_id):
    result = subprocess.run(['scancel', job_id], stdout=subprocess.PIPE)
    return result


# sacct may be severely out of sync (behind) actual state in slurm, so any code calling this method
# should be tolerant of getting an AttributeError and handling it gracefully. (either thrown
# explicitly or from the failed regex match).
@retry(AttributeError, tries=5, delay=1, backoff=2, logger=None)
def status_for_job_id(job_id):
    cmd_line = ' '.join(['sacct', '-X', '-P', '--format=jobid,state', '-j', job_id])
    logger.debug('Running command: %s' % cmd_line)
    result = subprocess.run(cmd_line.split(' '), stdout=subprocess.PIPE)
    result = result.stdout.decode('utf-8').strip()
    # might have > 1 line matching for a single job, so success only if all success, fail if 1+ fail,
    # otherwise running
    rows = result.split('\n')
    # There is a single output header row. There must be at least one data row after it to continue.
    if len(rows) < 2:
        logger.debug('sacct failed to return any matches for job id: %s' % job_id)
        raise AttributeError('No sacct entries found matching: %s' % job_id)
    rows = rows[1:]
    matches = [SACCT_REGEX.match(row) for row in rows]

    # this can also throw an AttributeError if the regex is broken
    states = {m.group('id'): m.group('state') for m in matches}
    # cancelled has trailing information, so correct that before mapping
    states = {id: v.split(' ')[0] if 'CANCELLED' in v else v for id, v in states.items()}

    mapped_states = {id: SLURM_STATES[state] for id, state in states.items()}

    ids_succeeded = [id for id, state in mapped_states.items() if state == EntityStatus.SUCCEEDED]
    ids_failed = [id for id, state in mapped_states.items() if state == EntityStatus.FAILED]
    ids_running = [id for id, state in mapped_states.items() if state == EntityStatus.RUNNING]

    if len(ids_failed) > 0:
        status = EntityStatus.FAILED
    elif len(ids_running) > 0:
        status = EntityStatus.RUNNING
    else:
        status = EntityStatus.SUCCEEDED
    logger.info('Status job id: %s: %s running, %s succeeded, %s failed' %
                (job_id, len(ids_running), len(ids_succeeded), len(ids_failed)))

    return status
